@@ -6,6 +6,48 @@ import requests
 import datetime
 import re
 import math
+from concurrent.futures import ThreadPoolExecutor, as_completed
+
+
+def fetch(client, method: str, params: dict = {}):
+    """ Execute method of a client with recived parameters """
+    method = getattr(client, method)
+    return method(**params)
+
+
+def bulk_get_pages(client, method, params: dict = {},
+        page_range: tuple = (1, 2), max_workers: int = 50, **kwargs) -> list:
+    """ Get a range of pages of a client simultaneously. """
+
+    # structures prepare
+    responses = []
+    processes = []
+    params_ = params
+
+    # create executor context
+    with ThreadPoolExecutor(max_workers=max_workers) as executor:
+
+        # processes prepare
+        for i in range(*page_range):
+            params_["params"]["page"] = i
+            processes.append(
+                executor.submit(
+                    fetch,
+                    client=client,
+                    method=method,
+                    params=params_
+                )
+            )
+
+        # execute and put result in out structure
+        [responses.append(r_.result()) for r_ in as_completed(processes)]
+
+        # close executor
+        executor.shutdown(wait=True)
+
+    # return responses
+    # list of pages result
+    return responses
 
 class JsonObject:
     """ Convert a dict['element'] for access like object.property. """
@@ -197,7 +239,7 @@ class Client:
             "url": urljoin(self.client_url.geturl(), path),
             "params": params,
             "headers": self.headers,
-            "timeout": kwargs.get("timeout", 20),
+            "timeout": kwargs.get("timeout", 60),
             "stream": kwargs.get("stream", False)
         }
 
@@ -341,7 +383,8 @@ class Client:
             "page": kwargs.get("page", None),
             "pageSize": kwargs.get("pageSize", 50),
             "active": kwargs.get("active", None),
-            "updatedFrom": kwargs.get("updatedFrom", None)
+            "updatedFrom": kwargs.get("updatedFrom", None),
+            "timeout": kwargs.get("timeout", 60)
         }
 
         # request.get -> json
@@ -353,13 +396,27 @@ class Client:
             num_pages = math.ceil(
                 response.get('totalCount') / params.get('pageSize')
             )
-            # recursive get and extend response values
-            for i in range(2, num_pages + 1):
-                # update page
-                params['page'] = i
-                response['values'].extend(
-                    self.get(path=path, params=params).get('values')
-                )
+
+            # get list of pages
+            pages_result = bulk_get_pages(
+                client=self,
+                method='get',
+                params={
+                    'path': path,
+                    'params': params
+                },
+                page_range=(2, num_pages + 1),
+                max_workers=kwargs.get('max_workers', 50)
+            )
+
+            # get values and add to structure
+            for pr in pages_result:
+                response['values'].extend(pr.get('values'))
+        
+        # put elements in dict if is ask extensions
+        if extension and kwargs.get('group_results', None):
+            return {employee: {extension: response}}
+
         # return elements
         return response
 
